import {
  select,
  csv,
  scaleOrdinal,
  pie,
  arc,
  format,
} from 'd3'; // Importing the D3 library

const svg = select('svg'); // Selecting the SVG element

const height = +svg.attr('height'); // Setting the height of the SVG
const width = +svg.attr('width'); // Setting the width of the SVG
const radius = Math.min(width, height) / 2.5; // Reduce the divisor to decrease the radius

// Color scale for different OS (using brighter colors)
const color = scaleOrdinal()
  .domain(['Android', 'iOS', 'BlackBerry OS', 'Sony'])
  .range(['#A4C639', '#007AFF', '#000000', '#c65b39']);

// Function to generate arc paths
const arcPath = arc()
  .outerRadius(radius - 10)
  .innerRadius(0)
  .cornerRadius(8);

// Render function to create the pie chart
const render = (data) => {
  const title = 'OS Users Data'; // Title of the visualization

  // Pie chart generator
  const pieChart = pie()
    .sort(null)
    .value((d) => d.value);

  // Generating arcs based on the pie chart data
  const arcs = pieChart(data);

  // Group element for the pie chart
  const g = svg.append('g').attr(
    'transform',
    `translate(${width / 2},${height / 2 + 50})`, // Shifted down by 50 units
  );

  // Adding pie slices to the pie chart
  g.selectAll('path')
    .data(arcs)
    .enter()
    .append('path')
    .attr('d', arcPath)
    .attr('fill', (d) => color(d.data.os))
    .each(function (d) {
      const center = arcPath.centroid(d);
      g.append('text')
        .attr('class', 'percentage')
        .attr('x', center[0])
        .attr('y', center[1])
        .attr('dy', '.35em')
        .text(
          format('.1%')(
            d.data.value / d3.sum(data, (d) => d.value),
          ),
        )
        .style('text-anchor', 'middle')
        .style('fill', 'white');
    });

  // Legend
  const legend = svg
    .append('g')
    .attr('class', 'legend')
    .attr(
      'transform',
      `translate(${width - 200},${height / 2 - 120})`,
    );

  const legendItems = legend
    .selectAll('.legend-item')
    .data(color.domain())
    .enter()
    .append('g')
    .attr('class', 'legend-item')
    .attr('transform', (d, i) => `translate(0,${i * 30})`);

  legendItems
    .append('rect')
    .attr('width', 20)
    .attr('height', 20)
    .attr('fill', color);

  legendItems
    .append('text')
    .attr('x', 30)
    .attr('y', 14)
    .text((d) => d)
    .attr('fill', '#333')
    .style('font-size', '14px');

  // Title
  svg
    .append('text')
    .attr('class', 'title')
    .attr('x', width / 2)
    .attr('y', 70)
    .attr('text-anchor', 'middle')
    .text(title);
};

// Load the data and render the pie chart
csv(
  'https://gist.githubusercontent.com/akhillenkalapalli/7712920150d2799aa628856a655dfbb6/raw/49bca5acf58fc235b9e206f3a4cbcd09818d230f/gistfile1.txt',
)
  .then((data) => {
    // Aggregate data for Android, iOS, and BlackBerry OS
    const aggregatedData = [
      {
        os: 'Android',
        value: d3.sum(data, (d) => +d.Android),
      },
      { os: 'iOS', value: d3.sum(data, (d) => +d.iOS) },
      {
        os: 'BlackBerry OS',
        value: d3.sum(data, (d) => +d['BlackBerry OS']),
      },
    ];
    return aggregatedData;
  })
  .then((aggregatedData) => {
    render(aggregatedData);
  });
